# Prevodjenje nukleotida u brojeve
def symbol_to_number(c):
    pairs = {
        'a' : 0,
        't' : 1,
        'c' : 2,
        'g' : 3
    }
    
    return pairs[c]

# Prevodjenje nukleotida u brojeve        
def number_to_symbol(n):
    pairs = {
        0 : 'a',
        1 : 't',
        2 : 'c',
        3 : 'g'
    }
    
    return pairs[n]

# Prevodjenje broja u odgovarajucu nukleotidnu sekvencu
def number_to_pattern(n, k):
    if k == 1:
        return number_to_symbol(n)
    
    prefix_index = n // 4
    r = n % 4
    c = number_to_symbol(r)
    prefix_pattern = number_to_pattern(prefix_index, k - 1)
    
    return prefix_pattern + c

# Prevodjenje nukleotidne sekvence u odgovarajuci broj
def pattern_to_number(pattern):
    if len(pattern) == 0:
        return 0
    
    last = pattern[-1:]
    prefix = pattern[:-1]
    
    return 4 * pattern_to_number(prefix) + symbol_to_number(last)
    

# Prebrojavanje pojavljivanja uzoraka duzine k u tekstu
def computing_frequencies(text, k):
    frequency_array = [0 for i in range(4**k)]
    
    for i in range(len(text) - k):
        pattern = text[i:i+k]
        j = pattern_to_number(pattern)
        frequency_array[j] += 1
    
    return frequency_array
        

# Pronalazenje sekvenci duzine k u tekstu koje se cesto pojavljuju
def faster_frequent_words(text, k):
    frequent_patterns = set([])
    
    frequency_array = computing_frequencies(text, k)
    
    max_count = max(frequency_array)
    
    for i in range(4**k):
        if frequency_array[i] == max_count:
            pattern = number_to_pattern(i, k)
            frequent_patterns.add(pattern)
    
    return (frequent_patterns, max_count)

def main():
    print(faster_frequent_words('atgcgctagtcactagtgtcagttcgatgtcgat', 3))
    
if __name__ == "__main__":
    main()
